@@ -12,7 +12,7 @@ use crate::{
         AnyDocumentId, BorrowedDocument, CollectionDocument, Document, DocumentId, KeyId,
         OwnedDocument, OwnedDocuments,
     },
-    key::Key,
+    key::{IntoPrefixRange, Key},
     schema::{CollectionName, Schematic},
     Error,
 };
@@ -234,7 +234,7 @@ where
 ///
 /// ```rust
 /// use bonsaidb_core::{
-///     schema::{Collection, CollectionName, DefaultSerialization, Schematic},
+///     schema::{Collection, DefaultSerialization, Schematic},
 ///     Error,
 /// };
 /// use serde::{Deserialize, Serialize};
@@ -395,6 +395,40 @@ pub trait SerializedCollection: Collection {
         ))
     }
 
+    /// Retrieves all documents with ids that start with `prefix`.
+    ///
+    /// ```rust
+    /// use bonsaidb_core::{
+    ///     connection::Connection,
+    ///     document::CollectionDocument,
+    ///     schema::{Collection, Schematic, SerializedCollection},
+    ///     Error,
+    /// };
+    /// use serde::{Deserialize, Serialize};
+    ///
+    /// #[derive(Debug, Serialize, Deserialize, Default, Collection)]
+    /// #[collection(name = "MyCollection", primary_key = String)]
+    /// # #[collection(core = bonsaidb_core)]
+    /// pub struct MyCollection;
+    ///
+    /// async fn starts_with_a<C: Connection>(
+    ///     db: &C,
+    /// ) -> Result<Vec<CollectionDocument<MyCollection>>, Error> {
+    ///     MyCollection::list_with_prefix(String::from("a"), db).await
+    /// }
+    /// ```
+    fn list_with_prefix<C>(prefix: Self::PrimaryKey, connection: &'_ C) -> List<'_, C, Self>
+    where
+        C: Connection,
+        Self: Sized,
+        Self::PrimaryKey: IntoPrefixRange,
+    {
+        List(connection::List::new(
+            connection::PossiblyOwned::Owned(connection.collection::<Self>()),
+            prefix.into_prefix_range().map(AnyDocumentId::Deserialized),
+        ))
+    }
+
     /// Retrieves all documents.
     ///
     /// ```rust
@@ -1202,6 +1236,7 @@ where
 impl<'a, Cn, Cl> List<'a, Cn, Cl>
 where
     Cl: Collection,
+    Cn: Connection,
 {
     /// Lists documents by id in ascending order.
     pub fn ascending(mut self) -> Self {
@@ -1220,6 +1255,30 @@ where
         self.0 = self.0.limit(maximum_results);
         self
     }
+
+    /// Returns the number of documents contained within the range.
+    ///
+    /// Order and limit are ignored if they were set.
+    ///
+    /// ```rust
+    /// # bonsaidb_core::__doctest_prelude!();
+    /// # fn test_fn<C: Connection>(db: &C) -> Result<(), Error> {
+    /// # tokio::runtime::Runtime::new().unwrap().block_on(async {
+    /// println!(
+    ///     "Number of documents with id 42 or larger: {}",
+    ///     MyCollection::list(42.., db).count().await?
+    /// );
+    /// println!(
+    ///     "Number of documents in MyCollection: {}",
+    ///     MyCollection::all(db).count().await?
+    /// );
+    /// # Ok(())
+    /// # })
+    /// # }
+    /// ```
+    pub async fn count(self) -> Result<u64, Error> {
+        self.0.count().await
+    }
 }
 
 impl<'a, Cn, Cl> Future for List<'a, Cn, Cl>
