@@ -107,7 +107,7 @@ impl Operation {
     }
 
     /// Inserts a new document with the serialized representation of `contents`
-    /// into `collection`.  If `id` is `None` a unique id will be generated. If
+    /// into `collection`. If `id` is `None` a unique id will be generated. If
     /// an id is provided and a document already exists with that id, a conflict
     /// error will be returned.
     pub fn insert_serialized<C: SerializedCollection>(
@@ -119,6 +119,23 @@ impl Operation {
         Ok(Self::insert(C::collection_name(), id, contents))
     }
 
+    /// Pushes a new document with the serialized representation of `contents`
+    /// into `collection`.
+    ///
+    /// ## Automatic Id Assignment
+    ///
+    /// This function calls [`SerializedCollection::natural_id()`] to try to
+    /// retrieve a primary key value from `contents`. If an id is returned, the
+    /// item is inserted with that id. If an id is not returned, an id will be
+    /// automatically assigned, if possible, by the storage backend, which uses
+    /// the [`Key`](crate::key::Key) trait to assign ids.
+    pub fn push_serialized<C: SerializedCollection>(contents: &C::Contents) -> Result<Self, Error> {
+        let id = C::natural_id(contents);
+        let id = id.map(DocumentId::new).transpose()?;
+        let contents = C::serialize(contents)?;
+        Ok(Self::insert(C::collection_name(), id, contents))
+    }
+
     /// Updates a document in `collection`.
     pub fn update(collection: CollectionName, header: Header, contents: impl Into<Bytes>) -> Self {
         Self {
@@ -269,7 +286,7 @@ pub struct Executed {
 #[derive(Clone, Debug, Serialize, Deserialize)]
 pub enum Changes {
     /// A list of changed documents.
-    Documents(Vec<ChangedDocument>),
+    Documents(DocumentChanges),
     /// A list of changed keys.
     Keys(Vec<ChangedKey>),
 }
@@ -278,9 +295,9 @@ impl Changes {
     /// Returns the list of documents changed in this transaction, or None if
     /// the transaction was not a document transaction.
     #[must_use]
-    pub fn documents(&self) -> Option<&[ChangedDocument]> {
-        if let Self::Documents(docs) = self {
-            Some(docs)
+    pub const fn documents(&self) -> Option<&DocumentChanges> {
+        if let Self::Documents(changes) = self {
+            Some(changes)
         } else {
             None
         }
@@ -298,11 +315,168 @@ impl Changes {
     }
 }
 
+/// A list of changed documents.
+#[derive(Clone, Debug, Serialize, Deserialize)]
+pub struct DocumentChanges {
+    /// All of the collections changed.
+    pub collections: Vec<CollectionName>,
+    /// The individual document changes.
+    pub documents: Vec<ChangedDocument>,
+}
+
+impl DocumentChanges {
+    /// Returns the changed document and the name of the collection the change
+    /// happened to.
+    #[must_use]
+    pub fn get(&self, index: usize) -> Option<(&CollectionName, &ChangedDocument)> {
+        self.documents.get(index).and_then(|doc| {
+            self.collections
+                .get(usize::from(doc.collection))
+                .map(|collection| (collection, doc))
+        })
+    }
+
+    /// Returns the number of changes in this collection.
+    #[must_use]
+    pub fn len(&self) -> usize {
+        self.documents.len()
+    }
+
+    /// Returns true if there are no changes.
+    #[must_use]
+    pub fn is_empty(&self) -> bool {
+        self.documents.is_empty()
+    }
+
+    /// Returns an interator over all of the changed documents.
+    pub const fn iter(&self) -> DocumentChangesIter<'_> {
+        DocumentChangesIter {
+            changes: self,
+            index: Some(0),
+        }
+    }
+}
+
+/// An iterator over [`DocumentChanges`].
+#[must_use]
+pub struct DocumentChangesIter<'a> {
+    changes: &'a DocumentChanges,
+    index: Option<usize>,
+}
+
+impl<'a> Iterator for DocumentChangesIter<'a> {
+    type Item = (&'a CollectionName, &'a ChangedDocument);
+
+    fn next(&mut self) -> Option<Self::Item> {
+        self.index.and_then(|index| {
+            let result = self.changes.get(index);
+            if result.is_some() {
+                self.index = index.checked_add(1);
+            }
+            result
+        })
+    }
+}
+
+/// A draining iterator over [`ChangedDocument`]s.
+#[must_use]
+pub struct DocumentChangesIntoIter {
+    collections: Vec<CollectionName>,
+    documents: std::vec::IntoIter<ChangedDocument>,
+}
+
+impl Iterator for DocumentChangesIntoIter {
+    type Item = (CollectionName, ChangedDocument);
+
+    fn next(&mut self) -> Option<Self::Item> {
+        self.documents.next().and_then(|doc| {
+            self.collections
+                .get(usize::from(doc.collection))
+                .map(|collection| (collection.clone(), doc))
+        })
+    }
+}
+
+impl IntoIterator for DocumentChanges {
+    type Item = (CollectionName, ChangedDocument);
+
+    type IntoIter = DocumentChangesIntoIter;
+
+    fn into_iter(self) -> Self::IntoIter {
+        DocumentChangesIntoIter {
+            collections: self.collections,
+            documents: self.documents.into_iter(),
+        }
+    }
+}
+
+#[test]
+fn document_changes_iter() {
+    let changes = DocumentChanges {
+        collections: vec![CollectionName::private("a"), CollectionName::private("b")],
+        documents: vec![
+            ChangedDocument {
+                collection: 0,
+                id: DocumentId::from_u64(0),
+                deleted: false,
+            },
+            ChangedDocument {
+                collection: 0,
+                id: DocumentId::from_u64(1),
+                deleted: false,
+            },
+            ChangedDocument {
+                collection: 1,
+                id: DocumentId::from_u64(2),
+                deleted: false,
+            },
+            ChangedDocument {
+                collection: 2,
+                id: DocumentId::from_u64(3),
+                deleted: false,
+            },
+        ],
+    };
+
+    assert_eq!(changes.len(), 4);
+    assert!(!changes.is_empty());
+
+    let mut a_changes = 0;
+    let mut b_changes = 0;
+    let mut ids = Vec::new();
+    for (collection, document) in changes.iter() {
+        assert!(!ids.contains(&document.id));
+        ids.push(document.id);
+        match collection.name.as_ref() {
+            "a" => a_changes += 1,
+            "b" => b_changes += 1,
+            _ => unreachable!("invalid collection name {collection}"),
+        }
+    }
+    assert_eq!(a_changes, 2);
+    assert_eq!(b_changes, 1);
+
+    let mut a_changes = 0;
+    let mut b_changes = 0;
+    let mut ids = Vec::new();
+    for (collection, document) in changes {
+        assert!(!ids.contains(&document.id));
+        ids.push(document.id);
+        match collection.name.as_ref() {
+            "a" => a_changes += 1,
+            "b" => b_changes += 1,
+            _ => unreachable!("invalid collection name {collection}"),
+        }
+    }
+    assert_eq!(a_changes, 2);
+    assert_eq!(b_changes, 1);
+}
+
 /// A record of a changed document.
 #[derive(Debug, Clone, Serialize, Deserialize)]
 pub struct ChangedDocument {
-    /// The id of the `Collection` of the changed `Document`.
-    pub collection: CollectionName,
+    /// The index of the `CollectionName` within the `collections` field of [`Changes::Documents`].
+    pub collection: u16,
 
     /// The id of the changed `Document`.
     pub id: DocumentId,
